""" Copyright 2008 Joseph Bergin
License: Creative Commons Attribution-Noncommercial-Share Alike 3.0 United States License

A demonstration of polymorphism and delegation
From Section 4.5 of the book
"""

from karel.robota import UrRobot
from karel.robota import North
from karel.robota import world
from tracer import Tracer

class BeeperPutter:
    "Interface for beeper putting robots"
    def distributeBeepers(self):
        raise NotImplementedError("Implemented in Subclasses")
    
class NoNeighbor(UrRobot, BeeperPutter):
    "A BeeperPutter that simply puts a beeper and moves"
    def distributeBeepers(self):
        self.putBeeper()
        self.move()
        
class NeighborTalker(UrRobot, BeeperPutter):
    "A BeeperPutter that puts a robot and moves, but also asks its neighbor to do the same"
    def __init__(self, street, avenue, direction, beepers, neighbor):
         UrRobot.__init__(self, street, avenue, direction, beepers)
         self.__neighbor = neighbor
         
    def distributeBeepers(self):
        self.putBeeper()
        self.move()
        self.__neighbor.distributeBeepers()
   
   
def task():
    noname = NoNeighbor(avenue = 1, street = 1, direction = North, beepers = 2)
    Tracer(noname, "charlie")
    noname = NeighborTalker(street= 1, avenue= 2, direction= North, beepers= 2, neighbor=noname)
    Tracer(noname, "martha")
    noname = NeighborTalker(1, 3, North, 2, noname)
    Tracer(noname, "sue")
    noname = NeighborTalker(1, 4, North, 2, noname)
    Tracer(noname, "driver")
    world.showWorldWithRobots(1, 1, 5, 5)
#    noname.setPausing(1)
    noname.distributeBeepers()
    noname.distributeBeepers()
    world.showWorldWithRobots(1, 1, 5, 5)
    noname.display()
    
if __name__ == '__main__' :    
    from karel.robota import window
    world.setSize(12,12)
    world.setDelay(20)
    window().run(task) #note the robot is created within the task

