""" Copyright 2008 Joseph Bergin
License: Creative Commons Attribution-Noncommercial-Share Alike 3.0 United States License

Implements the Choreographer from Chapter 4 of the text
    Note that this is just the simplest case where each of the dancers is just a 
    basic robot. It can get much more complex if some of them override some of the 
    basic methods. The choreographer can also send a different message than the one
    it received, so when it "moves" the dancers it directs can "twirl" or whatever. 
    A still more complex situation occurs when the choreographer maintains separate
    lists for different kinds of dancers and sends different messages to the dancers
    in the different lists. addMaleDancer, addFemaleDancer, ...
    
    This introduces lists, including the append method and the for (all) iterator
"""

from karel.robota import East

from tracer import Tracer

from karel.robota import world
from karel.robota import UrRobot
from harvester import Harvester
from copy import copy


class Choreographer(Harvester) :
    "Manage a team of dancers robots, each of which mimics the Choreographer"
    
    def __init__(self, street, avenue, direction, beepers):
        Harvester.__init__(self, street, avenue, direction, beepers)
        self.__dancers = []
        
    def clone(self):
        result = Harvester.clone(self)
        result.__dancers = []
        for robot in self.__dancers :
            result.__dancers.append(robot.clone())
        return result

#    def clone(self):
#        result = Harvester.clone(self)
#        result.__dancers = copy(self.__dancers)
#        return result
        
    def addDancer(self, dancer):
        "Add a dancer robot to the dance team"
        self.__dancers.append(dancer)
        
    def move(self):
        "Move the entire team forward"
#        UrRobot.move(self);
        super().move()
        for dancer in self.__dancers :
            dancer.move()
            
    def turnLeft(self):
        "Move the entire team forward"
        UrRobot.turnLeft(self);
        for dancer in self.__dancers :
            dancer.turnLeft()
            
    def pickBeeper(self):
        "Move the entire team forward"
#        UrRobot.pickBeeper(self);
        super().pickBeeper()
        for dancer in self.__dancers :
            dancer.pickBeeper()


    def putBeeper(self):
        "Move the entire team forward"
        UrRobot.putBeeper(self);
        for dancer in self.__dancers :
            dancer.putBeeper()
            
    def turnOff(self):
        "Move the entire team forward"
        UrRobot.turnOff(self);
        for dancer in self.__dancers :
            dancer.turnOff()
            
def task():
    world.readWorld("../fig3-2.kwld")
    world.setDelay(10)
    martha = Choreographer(2, 2, East, 0)
    d1 = UrRobot(4, 2, East, 0)
    martha.addDancer(d1)
    Tracer(d1, "Mary")
    d2 = UrRobot(6, 2, East, 0)
    martha.addDancer(d2)
#    Tracer(d2, "Oleg")
#    Tracer(martha, "Martha")
#    martha.move()
#    martha.harvestTwoRows()
    marsha = martha.clone()
    martha.turnOff()
    marsha.move()
    marsha.harvestTwoRows()
    marsha.turnLeft(); marsha.move()
#    martha.move()
#    martha.turnLeft()
#    martha.move()
    world.showWorldWithRobots(1, 1, 8, 6)
    
if __name__ == '__main__' :    
    from karel.robota import window
    world.setSize(12,12)
    world.setDelay(50)
    window().run(task) #note the robot is created within the task

