""" Copyright 2008 Joseph Bergin
License: Creative Commons Attribution-Noncommercial-Share Alike 3.0 United States License

Implements the Contractor from Chapter 4 of the text
    This code introduces lists and iterators over lists. 
"""

from karel.robota import East
from karel.robota import North

from karel.robota import world
from karel.robota import UrRobot
from karel.robota import infinity

class Worker:
    "interface for worker robots" 
    def getToWork(self):
        raise NotImplementedError('getToWork error')
    


class Contractor:
    "Manage a team of worker robots, each of which knows their job"
    
    def __init__(self):
#        UrRobot.__init__(self, street, avenue, direction, beepers)
        self.__workers = []
        
    def _gatherTeam(self):
#        self._addWorker(TwoLayer(2, 1, East, infinity))
#        self._addWorker(ThreeLayer(1, 1, East, infinity))
        self._addWorker(Carpenter(2, 1, East, infinity))
        self._addWorker(Mason(1, 1, East, infinity))
        self._addWorker(Roofer(1, 1, East, infinity))
       
        
    def _addWorker(self, worker):
        "Add a worker robot to the work team"
        self.__workers.append(worker)
        
    def startJob(self):
        "Put the entire team to work"
        self._gatherTeam()
        for worker in self.__workers :
            worker.getToWork()
        
class Carpenter(UrRobot, Worker):
    def getToWork(self):
        self._makeDoor()
        self._makeWindow()
        self._makeWindow()
        pass
    
    def _makeWindow(self):
        print ("window")
        pass
    
    def _makeDoor(self):
        print ("door")
        pass

    def _moveToFirstWindow(self):
        print ("moving")
    def _moveToNextWindow(self):
        print ( "moving")
    
class Roofer(UrRobot, Worker):
    def getToWork(self):
        self.makeRoof()
        
    def makeRoof(self):
        self._makeLeftGable()
        self._makeRightGable()
        
    def _makeLeftGable(self):
        print ( "left gable")
        pass
    
    def _makeRightGable(self):
        print ( "right gable")
        pass
    
class Mason(UrRobot, Worker):
    def getToWork(self):
        self._buildWall()
        self._moveToSecondWallBase()
        self._buildWall()
        
    def _buildWall(self):
        print ( "wall")
        pass
    
    def _moveToSecondWallBase(self):
        print ( "moving")
        pass

class TwoLayer(UrRobot, Worker):  
    def getToWork(self):
        self.putBeeper();
        self.putBeeper();
        self.move();
        self.putBeeper();
        self.putBeeper();
        self.move();
        
class ThreeLayer(UrRobot, Worker):
    def getToWork(self):
        self.putBeeper();
        self.putBeeper();
        self.putBeeper();
        self.move();
        self.putBeeper();
        self.putBeeper();
        self.putBeeper();
        self.move();
        
# From chapter 8

def carpenterTask() :
    linda =  Carpenter(1, 1, North, infinity);
    linda._moveToFirstWindow();
    linda._makeWindow();
    linda._moveToNextWindow();
    linda._makeWindow();
    
class CarpenterRunner :
    def run(self) :
        carpenterTask()


        
def task():
    kristen = Contractor()
    world.setDelay(5)
#    charlie.addWorker(TwoLayer(2, 1, East, infinity))
#    charlie.addWorker(ThreeLayer(1, 1, East, infinity))
    kristen.startJob()
    world.showWorldWithRobots(1, 1, 6, 6)
    
    carpenterTask()
    
    runner = CarpenterRunner()
    world.setupThread(runner)
    print ()
    world.startThreads(10)
                      

if __name__ == '__main__' :    
    from karel.robota import window
    world.setSize(12,12)
    window().run(task) #note the robot is created within the task
        
