''' Copyright 1997 Joseph Bergin
This may be reproduced, shared, modified for any educational purpose with attribution 
'''

#pg 209

from karel.robota import East
from karel.robota import North
from karel.robota import West
from karel.robota import South
from karel.robota import world
from default.turnermixin import Turner
from karel.robota import Robot
from random import uniform

 

class Die :
    """ A discrete, uniform random generator in range 1..max"""
    def __init__(self, max):
        ''' define the number of faces on the die'''
        self.__bound = max + 1
        
    def roll(self):
        ''' roll the die
        Note that uniform returns a floating number. We truncate it 
	to get an int
        '''
        return int(uniform(1, self.__bound))

class Philosopher(Robot, Turner) :
    def __init__(self, street, avenue,  direction) :
        Robot.__init__(self, street, avenue, direction, 0)
        world.setupThread(self)   
        self.__die = Die(6)
        
    def run(self) :
        while True :
            self._think(self.__die.roll())
            self._getForks()
            self._eat(self.__die.roll())
            self._putForks()
    
    def _think(self, time) : # move back to think
        for i in range(time) :
            self.backUp()
            self.move()
         
    def _eat(self, time) : # move to the center to eat
        self.userPause("eat");
        for i in range(time) :
            self.move()
            self.backUp()     
    
    def _getForks(self) :
        self.turnLeft()
        self.move()
        while not self.anyBeepersInBeeperBag() :
            while not self.nextToABeeper() :
                pass
            self.pickBeeper()
         
        self.turnAround()
        self.move()
        self.putBeeper()
        self.move()
        while not self.anyBeepersInBeeperBag() :
            while not self.nextToABeeper() :
                pass
            self.pickBeeper()
         
        self.turnAround()
        self.move()
        self.putBeeper()
        self.turnRight()
        self.showState("Eat ")
    
    def _putForks(self) :
        self.pickBeeper()
        self.pickBeeper()
        self.turnLeft()
        self.move()
        self.putBeeper()
        self.turnAround()
        self.move()
        self.move()
        self.putBeeper()
        self.turnAround()
        self.move()
        self.turnRight()
        self.showState("Think ")
     

if __name__ == '__main__' :
    world.setVisible(True)
    world.showSpeedControl(True) # Control the speed
    world.placeBeepers(2, 2, 1) # the forks
    world.placeBeepers(2, 4, 1)
    world.placeBeepers(4, 2, 1)
    world.placeBeepers(4, 4, 1)
    p1 = Philosopher(2, 3, North)
    p2 = Philosopher(3, 2, East)
    p3 = Philosopher(4, 3, South)
    p4 = Philosopher(3, 4, West)
    world.setDelay(10)
    world.startThreads(30)